@@ -3,18 +3,19 @@
     zeronimo
     ~~~~~~~~
 
-    :copyright: (c) 2013 by Heungsub Lee
+    :copyright: (c) 2013-2014 by Heungsub Lee
     :license: BSD, see LICENSE for more details.
 """
 from __future__ import absolute_import
 
 from .components import Worker, Customer, Collector, Task
 from .exceptions import (
-    ZeronimoException, WorkerNotFound, SocketClosed, UnexpectedMessage, raises)
+    ZeronimoException, WorkerNotFound, WorkerNotReachable,
+    TaskRejected, SocketClosed, MalformedMessage, raises)
 from .messaging import send, recv
 
 
-__version__ = '0.1.3'
+__version__ = '0.1.4'
 __all__ = ['Worker', 'Customer', 'Collector', 'Task', 'ZeronimoException',
            'WorkerNotFound', 'SocketClosed', 'UnexpectedMessage', 'raises',
-           'send', 'recv']
+           'send', 'recv', 'PACK', 'UNPACK']
