@@ -3,7 +3,7 @@
     zeronimo.exceptions
     ~~~~~~~~~~~~~~~~~~~
 
-    :copyright: (c) 2013 by Heungsub Lee
+    :copyright: (c) 2013-2014 by Heungsub Lee
     :license: BSD, see LICENSE for more details.
 """
 from contextlib import contextmanager
@@ -11,8 +11,8 @@ from contextlib import contextmanager
 import zmq
 
 
-__all__ = ['ZeronimoException', 'WorkerNotFound', 'UnexpectedMessage',
-           'make_worker_not_found', 'raises']
+__all__ = ['ZeronimoException', 'WorkerNotFound', 'WorkerNotReachable',
+           'TaskRejected', 'SocketClosed', 'MalformedMessage', 'raises']
 
 
 class ZeronimoException(BaseException):
@@ -21,7 +21,7 @@ class ZeronimoException(BaseException):
     pass
 
 
-class WorkerNotFound(ZeronimoException, LookupError):
+class WorkerNotFound(ZeronimoException):
     """Occurs by a collector which failed to find any worker accepted within
     the timeout.
     """
@@ -29,39 +29,30 @@ class WorkerNotFound(ZeronimoException, LookupError):
     pass
 
 
+class WorkerNotReachable(WorkerNotFound):
+    """The customer has no connection to any worker."""
+
+    pass
+
+
+class TaskRejected(WorkerNotFound):
+    """Failed to find worker accepts the task within the timeout."""
+
+    pass
+
+
 class SocketClosed(ZeronimoException, zmq.ZMQError):
     """Collector socket is closed while a task is collecting replies."""
 
     pass
 
 
-class UnexpectedMessage(ZeronimoException, RuntimeWarning):
+class MalformedMessage(ZeronimoException, RuntimeWarning):
     """Warns when a received message is not expected format."""
 
     pass
 
 
-def make_worker_not_found(rejected=0):
-    """Generates an error message by the count of workers rejected for
-    :exc:`WorkerNotFound`.
-
-        >>> make_worker_not_found(rejected=0)
-        WorkerNotFound('Worker not found',)
-        >>> make_worker_not_found(rejected=1)
-        WorkerNotFound('Worker not found, a worker rejected',)
-        >>> make_worker_not_found(rejected=10)
-        WorkerNotFound('Worker not found, 10 workers rejected',)
-    """
-    errmsg = ['Worker not found']
-    if rejected == 1:
-        errmsg.append('a worker rejected')
-    elif rejected:
-        errmsg.append('{0} workers rejected'.format(rejected))
-    exc = WorkerNotFound(', '.join(errmsg))
-    exc.rejected = rejected
-    return exc
-
-
 @contextmanager
 def raises(exctype):
     """Expected exceptions occured in the context will not raised at the
